<?php

namespace Drupal\eplikt\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\eplikt\EpliktSourcePluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Output e-Plikt all and weekly RSS feeds.
 */
class EpliktController extends ControllerBase {

  protected RendererInterface $renderer;
  protected ImmutableConfig $config;
  protected EpliktSourcePluginManager $pluginManager;
  protected LoggerChannelInterface $logger;

  /**
   * EpliktController constructor.
   */
  public function __construct(RendererInterface $renderer, ConfigFactoryInterface $config_factory, LoggerChannelFactoryInterface $logger_factory, EpliktSourcePluginManager $plugin_manager) {
    $this->renderer = $renderer;
    $this->config = $config_factory->get('eplikt.settings');
    $this->logger = $logger_factory->get('eplikt');
    $this->pluginManager = $plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('renderer'),
      $container->get('config.factory'),
      $container->get('logger.factory'),
      $container->get('plugin.manager.eplikt_source')
    );
  }

  /**
   * Get all entities.
   */
  public function all(): Response {
    $response = new Response();
    $response->headers->set('Content-Type', 'application/rss+xml');
    $entities = $this->getEntities();
    $items = $this->prepareItems($entities);
    $build = [
      '#theme' => 'eplikt_rss',
      'items' => $items,
    ];
    $content = $this->renderer->render($build);
    $response->setContent((string) $content);
    return $response;
  }

  /**
   * Get all entities updated within a week.
   */
  public function weekly(): Response {
    $response = new Response();
    $response->headers->set('Content-Type', 'application/rss+xml');
    $max_age = 60 * 60 * 24 * 7;
    $entities = $this->getEntities($max_age);
    $items = $this->prepareItems($entities);
    $build = [
      '#theme' => 'eplikt_rss',
      'items' => $items,
    ];
    $content = $this->renderer->renderInIsolation($build);
    $response->setContent((string) $content);
    return $response;
  }

  /**
   * Get entities for all enabled source plugins.
   *
   * @return ContentEntityInterface[]
   */
  protected function getEntities(?int $max_age = NULL): array {
    $sources = $this->getEnabledSources();
    $entities = [];
    /** @var \Drupal\eplikt\EpliktSourceInterface $source */
    foreach ($sources as $source) {
      $entities = array_merge($entities, $source->getEntities($max_age));
    }
    return $entities;
  }

  /**
   * Retrieves and initializes enabled source plugins from the configuration.
   *
   * @return array
   *   An array of initialized source plugin instances. If an exception occurs
   *   during initialization, the error is logged, and the source is excluded.
   */
  protected function getEnabledSources(): array {
    $sources = [];
    $enabled_sources = $this->config->get('sources');
    foreach ($enabled_sources as $key => $id) {
      try {
        $sources[] = $this->pluginManager->createInstance($id);
      }
      catch (\Exception $e) {
        $this->logger->error($e->getMessage());
      }
    }
    return $sources;
  }

  protected function prepareItems(array $entities): array {
    $items = [];
    foreach ($entities as $entity) {
      $build = [
        '#theme' => 'eplikt_rss_item',
        '#entity' => $entity,
        '#sorted' => TRUE,
      ];
      $items[] = $build;
    }
    return $items;
  }

}
