<?php

declare(strict_types=1);

namespace Drupal\eplikt\Form;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\eplikt\EpliktSourcePluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a settings form for the e-Plikt configuration.
 */
class SettingsForm extends ConfigFormBase {

  protected EpliktSourcePluginManager $pluginManager;

  /**
   * Constructs a new SettingsForm.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    protected TypedConfigManagerInterface $typedConfigManager,
    EpliktSourcePluginManager $plugin_manager
  ) {
    parent::__construct($config_factory, $this->typedConfigManager);
    $this->pluginManager = $plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('config.factory'),
      $container->get('config.typed'),
      $container->get('plugin.manager.eplikt_source')
    );
  }


  /**
   * {@inheritDoc}
   */
  public function getFormId(): string {
    return 'eplikt_settings';
  }

  /**
   * {@inheritDoc}
   */
  public function getEditableConfigNames(): array {
    return ['eplikt.settings'];
  }

  /**
   * {@inheritDoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form = parent::buildForm($form, $form_state);
    $config = $this->config('eplikt.settings');

    $form['publisher'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Publisher'),
      '#description' => $this->t('Unique identifier for the actor who has produced and/or made the resource available via the internet.'),
      '#default_value' => $config->get('publisher'),
    ];

    $form['access_rights'] = [
      '#type' => 'select',
      '#title' => $this->t('Access Rights'),
      '#description' => $this->t('Select the default access rights for the e-Plikt export.'),
      '#options' => [
        'gratis' => $this->t('Free'),
        'restricted' => $this->t('Restricted'),
      ],
      '#default_value' => $config->get('access_rights'),
    ];

    $sources = $config->get('sources');
    $form['sources'] = [
      '#type' => 'select',
      '#title' => $this->t('Sources'),
      '#description' => $this->t('Select the source plugins to use to collect data for the e-Plikt export.'),
      '#options' => $this->getPluginOptions(),
      '#multiple' => TRUE,
      '#default_value' => $sources,
    ];

    // Get the config form for each selected plugin.
    foreach ($sources as $id) {
      $plugin = $this->pluginManager->createInstance($id);
      $plugin->setConfiguration($config->get($id) ?? []);
      $form[$id] = [
        '#type' => 'details',
        '#title' => $this->t('@source source configuration', ['@source' => $plugin->getLabel()]),
        '#tree' => TRUE,
      ];

      $form[$id] += $plugin->buildConfigurationForm([], $form_state);
    }


    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $values = $form_state->getValues();
    $config = $this->configFactory()->getEditable('eplikt.settings');
    $config->set('access_rights', $values['access_rights'])
      ->set('publisher', $values['publisher'])
      ->set('sources', $values['sources']);
    foreach ($values['sources'] as $source_id) {
      $plugin_config = $form_state->getValue($source_id);
      foreach ($plugin_config as $key => $value) {
        $config->set($source_id . '.' . $key, $value);
      }
    }
    $config->save();
  }

  /**
   * Builds the plugin options.
   *
   * @return array
   *   An array of available plugins.
   * @throws PluginException
   */
  protected function getPluginOptions(): array {
    $options = [];
    $all_plugins = $this->pluginManager->getDefinitions();
    uasort($all_plugins, function ($a, $b) {
      return strnatcasecmp((string)$a['label'], (string)$b['label']);
    });
    foreach ($all_plugins as $definition) {
      /** @var \Drupal\eplikt\EpliktSourceInterface $plugin */
      $plugin = $this->pluginManager->createInstance($definition['id']);
      $options[$plugin->getPluginId()] = $plugin->getLabel();
    }
    return $options;
  }



}
