<?php

declare(strict_types=1);

namespace Drupal\eplikt\Hook;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Represents a central place to manage all theme-related hooks.
 */
class ThemeHooks implements ContainerInjectionInterface {

  use StringTranslationTrait;

  protected ConfigFactoryInterface $configFactory;
  protected RequestStack $requestStack;
  protected LanguageManagerInterface $languageManager;

  /**
   * ThemeHooks constructor.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    RequestStack $request_stack,
    LanguageManagerInterface $language_manager
  ) {
    $this->configFactory = $config_factory;
    $this->requestStack = $request_stack;
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('request_stack'),
      $container->get('language_manager')
    );
  }

  /**
   * Implements hook_theme().
   */
  #[Hook('theme')]
  public function theme($existing, $type, $theme, $path): array {
    return [
      'eplikt_rss' => [
        'render element' => 'items',
      ],
      'eplikt_rss_item' => [
        'variables' => [
          'entity' => NULL,
        ]
      ],
      // Specified here so that we can supply a default template for media entities.
      'eplikt_rss_item__media' => [
        'variables' => [
          'entity' => NULL,
        ]
      ]
    ];
  }

  /**
   * Implements hook_theme_suggestions_alter().
   */
  #[Hook('theme_suggestions_alter')]
  function themeSuggestionsAlter(array &$suggestions, array $variables, $hook) {
    if ($hook == 'eplikt_rss_item') {
      $entity = $variables['entity'];
      // TODO: Add suggestions based on plugin.
      $suggestions[] = $hook . '__' . $entity->getEntityTypeId();
      $suggestions[] = $hook . '__' . $entity->getEntityTypeId() . '__' . $entity->bundle();
    }
  }

  /**
   * Implements hook_preprocess_hook() for 'eplikt_rss'.
   *
   * This hook is not supported by the new hook annotation so it is invoked via the legacy hook system.
   */
  function preprocessEpliktRss(array &$variables) {
    $site_name = $this->configFactory->get('system.site')->get('name');
    $variables['base_url'] = $this->requestStack->getCurrentRequest()->getSchemeAndHttpHost();
    $variables['title'] = $this->t('e-Plikt | @site_name', ['@site_name' => $site_name]);
    $variables['language'] = $this->languageManager->getCurrentLanguage()->getId();
    $variables['description'] = $this->t('Legal deposit delivery for @site_name.', ['@site_name' => $site_name]);
  }

  /**
   * Implements hook_preprocess_hook() for 'eplikt_rss_item'.
   *
   * This hook is not supported by the new hook annotation so it is invoked via the legacy hook system.
   */
  function preprocessEpliktRssItem(array &$variables) {
    $entity = $variables['entity'];
    $variables['guid'] = $entity->uuid();
    $variables['link'] = $entity->toUrl()->toString();
    $variables['pubdate'] = date('r', (int)$entity->getCreatedTime());
    $variables['publisher'] = $this->configFactory->get('eplikt.settings')->get('publisher');
    $variables['title'] = $entity->label();
    $variables['access_rights'] = $this->configFactory->get('eplikt.settings')->get('access_rights');
    $variables['format'] = 'text/html';
  }

  /**
   * Implements hook_preprocess_hook() for 'eplikt_rss_item__media'.
   *
   * This hook is not supported by the new hook annotation so it is invoked via the legacy hook system.
   */
  function preprocessEpliktRssItemMedia(array &$variables) {
    $entity = $variables['entity'];
    $this->preprocessEpliktRssItem($variables);
    $variables['link'] = Url::fromRoute('media_entity_download.download', ['media' => $entity->id()], ['absolute' => TRUE])->toString();
    $fid = $entity->getSource()->getSourceFieldValue($entity);
    $file = File::load($fid);
    $variables['format'] = $file->getMimeType();
    $variables['media_link'] = $variables['link'];
    $variables['media_format'] = $variables['format'];
  }

}
