<?php

declare(strict_types=1);

namespace Drupal\eplikt\Plugin\eplikt;

use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\eplikt\EpliktSourceBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides e-Plikt node source plugin.
 *
 * @EpliktSource(
 *   id = "node_source",
 *   label = @Translation("Node"),
 *   description = @Translation("e-Plikt source for Node entities.")
 * )
 */
class NodeSource extends EpliktSourceBase {

  protected EntityTypeManagerInterface $entityTypeManager;
  protected EntityTypeBundleInfoInterface $entityTypeBundleInfo;

  /**
   * NodeSource constructor.
   */
  public function __construct(
    array $configuration,
          $plugin_id,
          $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    EntityTypeBundleInfoInterface $entity_type_bundle_info
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getEntities(?int $max_age = NULL): array {
    $query = $this->entityTypeManager->getStorage('node')->getQuery();
    $query->accessCheck(FALSE)
      ->condition('status', 1)
      ->sort('created', 'DESC');

    $bundles = $this->configuration['bundles'] ?? [];
    if (!empty($bundles)) {
      $query->condition('bundle', $this->configuration['bundles'], 'IN');
    }

    if (!is_null($max_age)) {
      $query->condition('changed', time() - $max_age, '>=');
    }

    $ids = $query->execute();
    return $this->entityTypeManager->getStorage('node')->loadMultiple($ids);
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state): array {
    $bundles = $this->entityTypeBundleInfo->getBundleInfo('node');
    $form['bundles'] = [
      '#type' => 'select',
      '#title' => $this->t('Bundles'),
      '#description' => $this->t('Select node bundles to include.'),
      '#options' => array_combine(array_keys($bundles), array_column($bundles, 'label')),
      '#multiple' => TRUE,
      '#default_value' => $this->configuration['bundles'] ?? NULL,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state): void {
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state): void {
    $this->configuration['bundles'] = $form_state->getValue('bundles');
  }

}
